@@ -1,4 +1,4 @@
-/* $Id: rmagick.h,v 1.37 2004/02/14 19:35:47 rmagick Exp $ */
+/* $Id: rmagick.h,v 1.75.2.1 2005/01/11 23:00:56 rmagick Exp $ */
 /*=============================================================================
 |               Copyright (C) 2004 by Timothy P. Hunter
 | Name:     rmagick.h
@@ -58,9 +58,11 @@
 // Safe replacement for rb_str2cstr
 #define STRING_PTR_LEN(v,l) rm_string_value_ptr_len(&(v), &(l))
 
+#undef DegreesToRadians     // defined in ImageMagick.h in 6.0.2
 #define DegreesToRadians(x) ((x)*3.14159265358979323846/180.0)
 
 typedef ImageInfo Info; // Make type name match class name
+typedef PixelPacket Pixel;
 
 // Montage
 typedef struct
@@ -70,11 +72,17 @@ typedef struct
 } Montage;
 
 // Draw
+struct TmpFile_Name
+{
+    struct TmpFile_Name *next;
+    char name[1];
+};
+
 typedef struct
 {
     DrawInfo *info;         // the DrawInfo struct
     VALUE primitives;       // the primitive string
-    VALUE tmpfile_ary;
+    struct TmpFile_Name *tmpfile_ary;
 } Draw;             // make the type match the class name
 
 // Enum
@@ -84,6 +92,8 @@ typedef struct
    int val;
 } MagickEnum;
 
+#undef False    // defined in deprecate.h in 6.0.2
+#undef True     // defined in deprecate.h in 6.0.2
 typedef enum
 {
     False = 0,
@@ -124,6 +134,28 @@ typedef struct
 
 #define MAX_GEOM_STR 51                 // max length of a geometry string
 
+#if defined(HAVE_QUANTUMOPERATORREGIONIMAGE) || defined(HAVE_EVALUATEIMAGECHANNEL)
+/*
+ * Both ImageMagick and GraphicsMagick define an enum type for quantum-level
+ * expressions, but they're different types. The QuantumExpressionOperator
+ * type is an adapter type that can be mapped to either one.
+*/
+typedef enum _QuantumExpressionOperator
+{
+    UndefinedQuantumOperator,
+    AddQuantumOperator,
+    AndQuantumOperator,
+    DivideQuantumOperator,
+    LShiftQuantumOperator,
+    MultiplyQuantumOperator,
+    OrQuantumOperator,
+    RShiftQuantumOperator,
+    SubtractQuantumOperator,
+    XorQuantumOperator
+} QuantumExpressionOperator ;
+#endif
+
+
 /*
     ImageMagick used simply size_t and off_t in 5.5.1, then defined the
     Extended(Un)SignedIntegralType from 5.5.2 thru 5.5.7. The 5.5.8 release
@@ -152,6 +184,10 @@ typedef size_t magick_uint64_t;
 #endif
 #endif
 
+#if !defined(HAVE_MAGICKBOOLEANTYPE)
+typedef unsigned int MagickBooleanType;
+#endif
+
 // This implements the "omitted storage class model" for external variables.
 // (Ref: Harbison & Steele.) The rmmain.c file defines MAIN, which causes
 // the single defining declarations to be generated. No other source files
@@ -180,6 +216,7 @@ EXTERN VALUE Class_Chromaticity;
 EXTERN VALUE Class_Color;
 EXTERN VALUE Class_Font;
 EXTERN VALUE Class_Geometry;
+EXTERN VALUE Class_GeometryValue;   // Defined in RMagick.rb
 EXTERN VALUE Class_Pixel;
 EXTERN VALUE Class_Point;
 EXTERN VALUE Class_Primary;
@@ -189,6 +226,13 @@ EXTERN VALUE Class_TypeMetric;
 #if defined(HAVE_COMPAREIMAGECHANNELS)
 EXTERN VALUE Class_MetricType;
 #endif
+#if defined(HAVE_QUANTUMOPERATORREGIONIMAGE) || defined(HAVE_EVALUATEIMAGECHANNEL)
+EXTERN VALUE Class_QuantumExpressionOperator;
+#endif
+#if defined(HAVE_GETIMAGESTATISTICS)
+EXTERN VALUE Class_Statistics;
+EXTERN VALUE Class_StatisticsChannel;
+#endif
 
 // Enum classes
 EXTERN VALUE Class_Enum;
@@ -202,6 +246,7 @@ EXTERN VALUE Class_CompositeOperator;
 EXTERN VALUE Class_CompressionType;
 EXTERN VALUE Class_DecorationType;
 EXTERN VALUE Class_DisposeType;
+EXTERN VALUE Class_EndianType;
 EXTERN VALUE Class_FilterTypes;
 EXTERN VALUE Class_GravityType;
 EXTERN VALUE Class_ImageType;
@@ -214,18 +259,36 @@ EXTERN VALUE Class_ResolutionType;
 EXTERN VALUE Class_StretchType;
 EXTERN VALUE Class_StyleType;
 EXTERN VALUE Class_WeightType;
+#if defined(HAVE_IMAGE_ORIENTATION)
+EXTERN VALUE Class_OrientationType;
+#endif
 
 /*
 *   Commonly-used IDs
 */
-EXTERN ID new_ID;          // "new"
-EXTERN ID push_ID;         // "push"
-EXTERN ID length_ID;       // "length"
-EXTERN ID values_ID;       // "values"
-EXTERN ID cur_image_ID;    // "cur_image"
-EXTERN ID call_ID;         // "call"
-EXTERN ID to_s_ID;         // "to_s"
-EXTERN ID _dummy_img__ID;  // "_dummy_img_"
+EXTERN ID ID__dummy_img_;       // "_dummy_img_"
+EXTERN ID ID_changed;           // "changed"
+EXTERN ID ID_call;              // "call"
+EXTERN ID ID_cur_image;         // "cur_image"
+EXTERN ID ID_dup;               // "dup"
+EXTERN ID ID_enumerators;       // "enumerators"
+EXTERN ID ID_fill;              // "fill"
+EXTERN ID ID_flag;              // "flag"
+EXTERN ID ID_from_s;            // "from_s"
+EXTERN ID ID_Geometry;          // "Geometry"
+EXTERN ID ID_GeometryValue;     // "GeometryValue"
+EXTERN ID ID_height;            // "height"
+EXTERN ID ID_initialize_copy;   // "initialize_copy"
+EXTERN ID ID_length;            // "length"
+EXTERN ID ID_notify_observers;  // "notify_observers"
+EXTERN ID ID_new;               // "new"
+EXTERN ID ID_push;              // "push"
+EXTERN ID ID_spaceship;         // "<=>
+EXTERN ID ID_to_s;              // "to_s"
+EXTERN ID ID_values;            // "values"
+EXTERN ID ID_width;             // "width"
+EXTERN ID ID_x;                 // "x"
+EXTERN ID ID_y;                 // "y"
 
 
 #if defined(HAVE_GETNEXTIMAGEINLIST)
@@ -254,9 +317,9 @@ EXTERN ID _dummy_img__ID;  // "_dummy_img_"
    ExceptionInfo structure (releasing any IM storage) before
    calling rb_raise.
 */
-#define HANDLE_ERROR handle_error(&exception);
+#define HANDLE_ERROR rm_handle_error(&exception);
 // For ExceptionInfo structures in images.
-#define HANDLE_IMG_ERROR(img) handle_error(&((img)->exception));
+#define HANDLE_ERROR_IMG(img) rm_handle_error(&((img)->exception));
 
 /*
     Map the QuantumDepth to a StorageType.
@@ -269,6 +332,7 @@ EXTERN ID _dummy_img__ID;  // "_dummy_img_"
 #define FIX_STG_TYPE LongPixel
 #endif
 
+
 /*
     Call rb_define_method for an attribute accessor method
 */
@@ -315,6 +379,7 @@ EXTERN ID _dummy_img__ID;  // "_dummy_img_"
     VALUE class##_##attr##_eq(VALUE self, VALUE val)\
     {\
         class *ptr;\
+        rm_check_frozen(self);\
         Data_Get_Struct(self, class, ptr);\
         ptr->attr = R_##type##_to_C_##type(val);\
         return self;\
@@ -334,15 +399,34 @@ EXTERN ID _dummy_img__ID;  // "_dummy_img_"
     ATTR_READER(class, attr)\
     ATTR_WRITER(class, attr)
 
+/*
+ *  Declare Pixel channel attribute writers
+*/
+#define DEF_PIXEL_CHANNEL_WRITER(_channel_) \
+extern VALUE \
+Pixel_##_channel_##_eq(VALUE self, VALUE v) \
+{ \
+    Pixel *pixel; \
+ \
+    rm_check_frozen(self); \
+    Data_Get_Struct(self, Pixel, pixel); \
+    pixel->_channel_ = (Quantum) NUM2UINT(v); \
+    rb_funcall(self, ID_changed, 0); \
+    rb_funcall(self, ID_notify_observers, 1, self); \
+    return INT2NUM(pixel->_channel_); \
+}
+
+
 /*
  *  Enum constants - define a subclass of Enum for the specified enumeration.
  *  Define an instance of the subclass for each member in the enumeration.
  *  Initialize each instance with its name and value.
  */
-#define DEF_ENUM(type) {\
+#define DEF_ENUM(tag) {\
    VALUE _cls, _enum;\
-   _cls =  Class_##type = rb_define_class_under(Module_Magick, #type, Class_Enum);
-#define ENUM_VAL(val)\
+   _cls =  Class_##tag = rm_define_enum_type(#tag);
+
+#define ENUMERATOR(val)\
    _enum = rm_enum_new(_cls, ID2SYM(rb_intern(#val)), INT2FIX(val));\
    rb_define_const(Module_Magick, #val, _enum);
 #define END_ENUM }
@@ -387,13 +471,17 @@ ATTR_WRITER(Draw, gravity)
 ATTR_WRITER(Draw, pointsize)
 ATTR_WRITER(Draw, rotation)
 ATTR_WRITER(Draw, stroke)
+ATTR_WRITER(Draw, stroke_width)
 ATTR_WRITER(Draw, text_antialias)
 ATTR_WRITER(Draw, undercolor)
 extern VALUE Draw_annotate(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Draw_clone(VALUE);
 extern VALUE Draw_composite(int, VALUE *, VALUE);
 extern VALUE Draw_draw(VALUE, VALUE);
 extern VALUE Draw_dup(VALUE);
+extern VALUE Draw_get_multiline_type_metrics(int, VALUE *, VALUE);
 extern VALUE Draw_get_type_metrics(int, VALUE *, VALUE);
+extern VALUE Draw_init_copy(VALUE, VALUE);
 extern VALUE Draw_initialize(VALUE);
 extern VALUE Draw_inspect(VALUE);
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
@@ -450,8 +538,11 @@ extern VALUE ImageList_to_blob(VALUE);
 extern VALUE ImageList_write(VALUE, VALUE);
 
 extern VALUE rm_imagelist_new(void);
+extern VALUE rm_imagelist_from_images(Image *);
+extern Image *rm_images_from_imagelist(VALUE);
+extern VALUE rm_imagelist_scene_eq(VALUE, VALUE);
 extern int rm_imagelist_length(VALUE);
-extern VALUE rm_imagelist_push(VALUE, VALUE);
+extern void rm_imagelist_push(VALUE, VALUE);
 
 
 // rminfo.c
@@ -521,6 +612,7 @@ ATTR_ACCESSOR(Image, density)
 ATTR_READER(Image, depth)
 ATTR_READER(Image, directory)
 ATTR_ACCESSOR(Image, dispose)
+ATTR_ACCESSOR(Image, endian)
 ATTR_ACCESSOR(Image, extract_info)
 ATTR_READER(Image, filename)
 ATTR_READER(Image, filesize)
@@ -543,7 +635,9 @@ ATTR_READER(Image, normalized_maximum_error)
 ATTR_READER(Image, number_colors)
 ATTR_ACCESSOR(Image, offset)
 ATTR_WRITER(Image, opacity)
+ATTR_READER(Image, orientation)
 ATTR_ACCESSOR(Image, page)
+ATTR_READER(Image, quality)
 ATTR_READER(Image, quantum_depth)
 ATTR_ACCESSOR(Image, rendering_intent)
 ATTR_READER(Image, rows)
@@ -573,29 +667,33 @@ extern VALUE Image_properties(VALUE);
 extern VALUE Image_bilevel_channel(int, VALUE *, VALUE);
 extern VALUE Image_black_threshold(int, VALUE *, VALUE);
 extern VALUE Image_blur_image(int, VALUE *, VALUE);
+extern VALUE Image_blur_channel(int, VALUE *, VALUE);
 extern VALUE Image_border(VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_border_bang(VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_capture(int, VALUE *, VALUE);
 extern VALUE Image_change_geometry(VALUE, VALUE);
 extern VALUE Image_changed_q(VALUE);
 extern VALUE Image_channel(VALUE, VALUE);
-extern VALUE Image_channel_compare(int, VALUE *, VALUE);
+extern VALUE Image_compare_channel(int, VALUE *, VALUE);
 extern VALUE Image_channel_depth(int, VALUE *, VALUE);
 extern VALUE Image_channel_extrema(int, VALUE *, VALUE);
 extern VALUE Image_channel_mean(int, VALUE *, VALUE);
 extern VALUE Image_channel_threshold(int, VALUE *, VALUE);
 extern VALUE Image_charcoal(int, VALUE *, VALUE);
 extern VALUE Image_chop(VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_clone(VALUE);
 extern VALUE Image_color_flood_fill(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_color_histogram(VALUE);
 extern VALUE Image_colorize(int, VALUE *, VALUE);
 extern VALUE Image_colormap(int, VALUE *, VALUE);
 extern VALUE Image_composite(int, VALUE *, VALUE);
+extern VALUE Image_composite_bang(int, VALUE *, VALUE);
 extern VALUE Image_composite_affine(VALUE, VALUE, VALUE);
 extern VALUE Image_compress_colormap_bang(VALUE);
 extern VALUE Image_constitute(VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_contrast(int, VALUE *, VALUE);
 extern VALUE Image_convolve(VALUE, VALUE, VALUE);
+extern VALUE Image_convolve_channel(int, VALUE *, VALUE);
 extern VALUE Image_copy(VALUE);
 extern VALUE Image_crop(int, VALUE *, VALUE);
 extern VALUE Image_crop_bang(int, VALUE *, VALUE);
@@ -605,6 +703,7 @@ extern VALUE Image_difference(VALUE, VALUE);
 extern VALUE Image_dispatch(int, VALUE *, VALUE);
 extern VALUE Image_display(VALUE);
 extern VALUE Image__dump(VALUE, VALUE);
+extern VALUE Image_dup(VALUE);
 extern VALUE Image_each_profile(VALUE);
 extern VALUE Image_edge(int, VALUE *, VALUE);
 extern VALUE Image_emboss(int, VALUE *, VALUE);
@@ -621,10 +720,13 @@ extern VALUE Image_from_blob(VALUE, VALUE);
 extern VALUE Image_gamma_channel(int, VALUE *, VALUE);
 extern VALUE Image_gamma_correct(int, VALUE *, VALUE);
 extern VALUE Image_gaussian_blur(int, VALUE *, VALUE);
+extern VALUE Image_gaussian_blur_channel(int, VALUE *, VALUE);
 extern VALUE Image_get_pixels(VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_gray_q(VALUE);
+extern VALUE Image_grayscale_pseudo_class(int, VALUE *, VALUE);
 extern VALUE Image_implode(int, VALUE *, VALUE);
 extern VALUE Image_import_pixels(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_init_copy(VALUE, VALUE);
 extern VALUE Image_inspect(VALUE);
 extern VALUE Image_level(int, VALUE *, VALUE);
 extern VALUE Image_level_channel(int, VALUE *, VALUE);
@@ -642,6 +744,7 @@ extern VALUE Image_motion_blur(VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_negate(int, VALUE *, VALUE);
 extern VALUE Image_negate_channel(int, VALUE *, VALUE);
 extern VALUE Image_normalize(VALUE);
+extern VALUE Image_normalize_channel(int, VALUE *, VALUE);
 extern VALUE Image_oil_paint(int, VALUE *, VALUE);
 extern VALUE Image_opaque(VALUE, VALUE, VALUE);
 extern VALUE Image_opaque_q(VALUE);
@@ -650,13 +753,18 @@ extern VALUE Image_palette_q(VALUE);
 extern VALUE Image_ping(VALUE, VALUE);
 extern VALUE Image_pixel_color(int, VALUE *, VALUE);
 // VALUE Image_plasma(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_posterize(int, VALUE *, VALUE);
 extern VALUE Image_preview(VALUE, VALUE);
 extern VALUE Image_profile_bang(VALUE, VALUE, VALUE);
 extern VALUE Image_quantize(int, VALUE *, VALUE);
 extern VALUE Image_quantization_error(VALUE);
+extern VALUE Image_quantum_operator(int, VALUE *, VALUE);
+extern VALUE Image_radial_blur(VALUE, VALUE);
 extern VALUE Image_raise(int, VALUE *, VALUE);
 extern VALUE Image_random_channel_threshold(VALUE, VALUE, VALUE);
+extern VALUE Image_random_threshold_channel(int, VALUE *, VALUE);
 extern VALUE Image_read(VALUE, VALUE);
+extern VALUE Image_read_inline(VALUE, VALUE);
 extern VALUE Image_reduce_noise(VALUE, VALUE);
 extern VALUE Image_resize(int, VALUE *, VALUE);
 extern VALUE Image_resize_bang(int, VALUE *, VALUE);
@@ -668,15 +776,19 @@ extern VALUE Image_sample_bang(int, VALUE *, VALUE);
 extern VALUE Image_scale(int, VALUE *, VALUE);
 extern VALUE Image_scale_bang(int, VALUE *, VALUE);
 extern VALUE Image_segment(int, VALUE *, VALUE);
+extern VALUE Image_set_channel_depth(VALUE, VALUE, VALUE);
 extern VALUE Image_shade(int, VALUE *, VALUE);
 extern VALUE Image_sharpen(int, VALUE *, VALUE);
+extern VALUE Image_sharpen_channel(int, VALUE *, VALUE);
 extern VALUE Image_shave(VALUE, VALUE, VALUE);
 extern VALUE Image_shave_bang(VALUE, VALUE, VALUE);
 extern VALUE Image_shear(VALUE, VALUE, VALUE);
 extern VALUE Image_signature(VALUE);
 extern VALUE Image_solarize(int, VALUE *, VALUE);
 extern VALUE Image_spaceship(VALUE, VALUE);
+extern VALUE Image_splice(int, VALUE *, VALUE);
 extern VALUE Image_spread(int, VALUE *, VALUE);
+extern VALUE Image_statistics(VALUE);
 extern VALUE Image_stegano(VALUE, VALUE, VALUE);
 extern VALUE Image_stereo(VALUE, VALUE);
 extern VALUE Image_store_pixels(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
@@ -690,7 +802,10 @@ extern VALUE Image_tint(int, VALUE *, VALUE);
 extern VALUE Image_to_blob(VALUE);
 extern VALUE Image_to_color(VALUE, VALUE);
 extern VALUE Image_transparent(int, VALUE *, VALUE);
-extern VALUE Image_unsharp_mask(VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_trim(VALUE);
+extern VALUE Image_trim_bang(VALUE);
+extern VALUE Image_unsharp_mask(int, VALUE *, VALUE);
+extern VALUE Image_unsharp_mask_channel(int, VALUE *, VALUE);
 extern VALUE Image_wave(int, VALUE *, VALUE);
 extern VALUE Image_white_threshold(int, VALUE *, VALUE);
 extern VALUE Image_write(VALUE, VALUE);
@@ -700,101 +815,132 @@ extern VALUE rm_image_new(Image *);
 
 // rmfill.c
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
-VALUE GradientFill_alloc(VALUE);
+extern VALUE  GradientFill_alloc(VALUE);
 #else
-VALUE GradientFill_new(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE  GradientFill_new(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
 #endif
 
-extern VALUE GradientFill_initialize(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
-extern VALUE GradientFill_fill(VALUE, VALUE);
+extern VALUE  GradientFill_initialize(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE  GradientFill_fill(VALUE, VALUE);
 
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
-VALUE TextureFill_alloc(VALUE);
+extern VALUE  TextureFill_alloc(VALUE);
 #else
-VALUE TextureFill_new(VALUE, VALUE);
+extern VALUE  TextureFill_new(VALUE, VALUE);
 #endif
 
-extern VALUE TextureFill_initialize(VALUE, VALUE);
-extern VALUE TextureFill_fill(VALUE, VALUE);
+extern VALUE  TextureFill_initialize(VALUE, VALUE);
+extern VALUE  TextureFill_fill(VALUE, VALUE);
 
 
 // rmutil.c
-extern VALUE   AffineMatrix_from_AffineMatrix(AffineMatrix *);
-extern VALUE   ChromaticityInfo_to_s(VALUE);
-extern VALUE   ChromaticityInfo_new(ChromaticityInfo *);
-extern void    Color_to_PixelPacket(PixelPacket *, VALUE);
-extern VALUE   Color_to_s(VALUE);
-extern VALUE   Color_from_ColorInfo(const ColorInfo *);
-extern VALUE   ColorspaceType_new(ColorspaceType);
-extern VALUE   CompressionType_new(CompressionType);
-extern VALUE   FilterTypes_new(FilterTypes);
-extern VALUE   Font_to_s(VALUE);
-extern VALUE   ImageList_cur_image(VALUE);
-extern VALUE   ImageMagickError_initialize(VALUE, VALUE, VALUE);
-extern VALUE   ImageType_new(ImageType);
-extern VALUE   InterlaceType_new(InterlaceType);
-extern VALUE   Pixel_fcmp(int, VALUE *, VALUE);
-extern VALUE   Pixel_from_color(VALUE, VALUE);
-extern VALUE   Pixel_from_HSL(VALUE, VALUE);
-extern VALUE   Pixel_intensity(VALUE);
-extern VALUE   Pixel_to_color(int, VALUE *, VALUE);
-extern VALUE   Pixel_to_HSL(VALUE);
-extern VALUE   Pixel_to_s(VALUE);
-extern VALUE   PixelPacket_to_Color_Name(Image *, PixelPacket *);
-extern VALUE   PixelPacket_to_Color_Name_Info(Info *, PixelPacket *);
-extern VALUE   Pixel_from_PixelPacket(PixelPacket *);
-extern void    Point_to_PointInfo(PointInfo *, VALUE);
-extern VALUE   PointInfo_to_Point(PointInfo *);
-extern VALUE   PrimaryInfo_to_s(VALUE);
-extern VALUE   PrimaryInfo_from_PrimaryInfo(PrimaryInfo *);
-extern VALUE   RectangleInfo_to_s(VALUE);
-extern VALUE   Rectangle_from_RectangleInfo(RectangleInfo *);
-extern VALUE   RenderingIntent_new(RenderingIntent);
-extern VALUE   ResolutionType_new(ResolutionType);
-extern VALUE   SegmentInfo_to_s(VALUE);
-extern VALUE   Segment_from_SegmentInfo(SegmentInfo *);
-extern void    AffineMatrix_to_AffineMatrix(AffineMatrix *, VALUE);
-extern void    ChromaticityInfo_to_ChromaticityInfo(ChromaticityInfo *, VALUE);
-extern void    Color_to_ColorInfo(ColorInfo *, VALUE);
-extern void    Pixel_to_PixelPacket(PixelPacket *, VALUE);
-extern void    PrimaryInfo_to_PrimaryInfo(PrimaryInfo *, VALUE);
-extern void    Rectangle_to_RectangleInfo(RectangleInfo *, VALUE);
-extern void    Segment_to_SegmentInfo(SegmentInfo *, VALUE);
-extern void    Font_to_TypeInfo(TypeInfo *, VALUE);
-extern void    TypeMetric_to_TypeMetric(TypeMetric *, VALUE);
-extern VALUE   Font_from_TypeInfo(TypeInfo *);
-extern VALUE   TypeMetric_to_s(VALUE);
-extern VALUE   TypeMetric_from_TypeMetric(TypeMetric *);
+extern VALUE  ChromaticityInfo_to_s(VALUE);
+extern VALUE  ChromaticityInfo_new(ChromaticityInfo *);
+extern void   Color_to_PixelPacket(PixelPacket *, VALUE);
+extern VALUE  Color_to_s(VALUE);
+extern VALUE  Color_from_ColorInfo(const ColorInfo *);
+extern VALUE  ClassType_new(ClassType);
+extern VALUE  ColorspaceType_new(ColorspaceType);
+extern VALUE  CompressionType_new(CompressionType);
+extern VALUE  EndianType_new(EndianType);
+extern VALUE  FilterTypes_new(FilterTypes);
+extern VALUE  Font_to_s(VALUE);
+extern VALUE  ImageList_cur_image(VALUE);
+extern VALUE  ImageMagickError_initialize(int, VALUE *, VALUE);
+extern VALUE  ImageType_new(ImageType);
+extern VALUE  InterlaceType_new(InterlaceType);
 
-extern VALUE   rm_enum_new(VALUE, VALUE, VALUE);
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
-extern VALUE Enum_alloc(VALUE);
+extern VALUE   Pixel_alloc(VALUE);
 #else
-extern VALUE Enum_new(VALUE, VALUE, VALUE);
+extern VALUE   Pixel_new(int, VALUE *, VALUE);
 #endif
-extern VALUE   Enum_initialize(VALUE, VALUE, VALUE);
-extern VALUE   Enum_to_s(VALUE);
-extern VALUE   Enum_to_i(VALUE);
-extern VALUE   Enum_spaceship(VALUE, VALUE);
-extern VALUE   Enum_case_eq(VALUE, VALUE);
+ATTR_ACCESSOR(Pixel, red)
+ATTR_ACCESSOR(Pixel, green)
+ATTR_ACCESSOR(Pixel, blue)
+ATTR_ACCESSOR(Pixel, opacity)
+extern VALUE  Pixel_case_eq(VALUE, VALUE);
+extern VALUE  Pixel_clone(VALUE);
+extern VALUE  Pixel_dup(VALUE);
+extern VALUE  Pixel_fcmp(int, VALUE *, VALUE);
+extern VALUE  Pixel_from_color(VALUE, VALUE);
+extern VALUE  Pixel_from_HSL(VALUE, VALUE);
+extern VALUE  Pixel_initialize(int, VALUE *, VALUE);
+extern VALUE  Pixel_init_copy(VALUE, VALUE);
+extern VALUE  Pixel_intensity(VALUE);
+extern VALUE  Pixel_spaceship(VALUE, VALUE);
+extern VALUE  Pixel_to_color(int, VALUE *, VALUE);
+extern VALUE  Pixel_to_HSL(VALUE);
+extern VALUE  Pixel_to_s(VALUE);
+extern VALUE  PixelPacket_to_Color_Name(Image *, PixelPacket *);
+extern VALUE  PixelPacket_to_Color_Name_Info(Info *, PixelPacket *);
+extern VALUE  Pixel_from_PixelPacket(PixelPacket *);
 
+extern void   Point_to_PointInfo(PointInfo *, VALUE);
+extern VALUE  PointInfo_to_Point(PointInfo *);
+extern VALUE  PrimaryInfo_to_s(VALUE);
+extern VALUE  PrimaryInfo_from_PrimaryInfo(PrimaryInfo *);
+extern VALUE  RectangleInfo_to_s(VALUE);
+extern VALUE  Rectangle_from_RectangleInfo(RectangleInfo *);
+extern VALUE  RenderingIntent_new(RenderingIntent);
+extern VALUE  ResolutionType_new(ResolutionType);
+extern VALUE  SegmentInfo_to_s(VALUE);
+extern VALUE  Segment_from_SegmentInfo(SegmentInfo *);
+extern void   AffineMatrix_to_AffineMatrix(AffineMatrix *, VALUE);
+extern void   ChromaticityInfo_to_ChromaticityInfo(ChromaticityInfo *, VALUE);
+extern void   Color_to_ColorInfo(ColorInfo *, VALUE);
+extern void   PrimaryInfo_to_PrimaryInfo(PrimaryInfo *, VALUE);
+extern void   Rectangle_to_RectangleInfo(RectangleInfo *, VALUE);
+extern void   Segment_to_SegmentInfo(SegmentInfo *, VALUE);
+extern void   Font_to_TypeInfo(TypeInfo *, VALUE);
+extern void   TypeMetric_to_TypeMetric(TypeMetric *, VALUE);
+extern VALUE  Font_from_TypeInfo(TypeInfo *);
+extern VALUE  TypeMetric_to_s(VALUE);
+extern VALUE  TypeMetric_from_TypeMetric(TypeMetric *);
+#if defined(HAVE_GETIMAGESTATISTICS)
+extern VALUE  Statistics_new(ImageStatistics *);
+#endif
+#if defined(HAVE_IMAGE_ORIENTATION)
+extern VALUE   OrientationType_new(OrientationType);
+#endif
+
+#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
+extern VALUE  Enum_alloc(VALUE);
+#else
+extern VALUE  Enum_new(VALUE, VALUE, VALUE);
+#endif
+extern VALUE  Enum_initialize(VALUE, VALUE, VALUE);
+extern VALUE  Enum_to_s(VALUE);
+extern VALUE  Enum_to_i(VALUE);
+extern VALUE  Enum_spaceship(VALUE, VALUE);
+extern VALUE  Enum_case_eq(VALUE, VALUE);
+extern VALUE  Enum_type_initialize(VALUE, VALUE, VALUE);
+extern VALUE  Enum_type_each(VALUE);
+
+
+extern void  *magick_malloc(const size_t);
+extern void   magick_free(void *);
+extern void  *magick_realloc(void *, size_t);
+extern void   magick_clone_string(char **, const char *);
+extern VALUE  rm_enum_new(VALUE, VALUE, VALUE);
+extern VALUE  rm_no_freeze(VALUE);
 #if !defined(StringValuePtr)
-extern char *rm_string_value_ptr(volatile VALUE *);
+extern char  *rm_string_value_ptr(volatile VALUE *);
 #endif
-extern char *rm_string_value_ptr_len(volatile VALUE *, long *);
-
-extern void *magick_malloc(const size_t);
-extern void magick_free(void *);
-extern void *magick_realloc(void *, size_t);
-extern void magick_clone_string(char **, const char *);
-extern void write_temp_image(Image *, char *);
-extern void delete_temp_image(char *);
-extern void not_implemented(const char *);
-extern void handle_error(ExceptionInfo *);
-extern void handle_all_errors(Image *);
-extern void attr_write(VALUE, VALUE);
-extern Image *toseq(VALUE);
-extern void unseq(Image *);
+extern char  *rm_string_value_ptr_len(volatile VALUE *, long *);
+extern void   rm_check_ary_len(VALUE, int);
+extern void   rm_check_frozen(VALUE);
+extern VALUE  rm_obj_to_s(VALUE);
+extern double rm_fuzz_to_dbl(VALUE fuzz);
+extern VALUE  rm_define_enum_type(char *);
+extern void   rm_write_temp_image(Image *, char *);
+extern void   rm_delete_temp_image(char *);
+extern void   rm_not_implemented(void);
+extern void   rm_handle_error(ExceptionInfo *);
+extern void   rm_handle_all_errors(Image *);
+extern void   rm_attr_write(VALUE, VALUE);
+extern void   rm_get_geometry(VALUE, long *, long *, unsigned long *, unsigned long *, int *);
+extern void   rm_split(Image *);
 
 #endif
 
