@@ -1,4 +1,4 @@
-/* $Id: rmdraw.c,v 1.8 2004/01/31 18:34:05 rmagick Exp $ */
+/* $Id: rmdraw.c,v 1.21 2004/12/17 22:42:51 rmagick Exp $ */
 /*============================================================================\
 |                Copyright (C) 2004 by Timothy P. Hunter
 | Name:     rmdraw.c
@@ -8,11 +8,16 @@
 \============================================================================*/
 
 #include "rmagick.h"
+#include "float.h"
 
 static void mark_Draw(void *);
 static void destroy_Draw(void *);
 static void destroy_Montage(void *);
 
+typedef MagickBooleanType (get_type_metrics_func_t)(Image *, const DrawInfo *, TypeMetric *);
+static VALUE get_type_metrics(int, VALUE *, VALUE, get_type_metrics_func_t *);
+
+
 /*
     Method:     Draw#affine=
     Purpose:    set the affine matrix from an Magick::AffineMatrix
@@ -22,6 +27,7 @@ Draw_affine_eq(VALUE self, VALUE matrix)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     AffineMatrix_to_AffineMatrix(&draw->info->affine, matrix);
     return self;
@@ -36,6 +42,7 @@ Draw_align_eq(VALUE self, VALUE align)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(align, draw->info->align, AlignType);
     return self;
@@ -50,6 +57,7 @@ Draw_decorate_eq(VALUE self, VALUE decorate)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(decorate, draw->info->decorate, DecorationType);
     return self;
@@ -64,6 +72,7 @@ Draw_density_eq(VALUE self, VALUE density)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     magick_clone_string(&draw->info->density, STRING_PTR(density));
 
@@ -79,6 +88,7 @@ Draw_encoding_eq(VALUE self, VALUE encoding)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     magick_clone_string(&draw->info->encoding, STRING_PTR(encoding));
 
@@ -94,6 +104,7 @@ Draw_fill_eq(VALUE self, VALUE fill)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     Color_to_PixelPacket(&draw->info->fill, fill);
     return self;
@@ -108,6 +119,7 @@ Draw_font_eq(VALUE self, VALUE font)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     magick_clone_string(&draw->info->font, STRING_PTR(font));
 
@@ -123,6 +135,7 @@ Draw_font_family_eq(VALUE self, VALUE family)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     magick_clone_string(&draw->info->family, STRING_PTR(family));
 
@@ -138,6 +151,7 @@ Draw_font_stretch_eq(VALUE self, VALUE stretch)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(stretch, draw->info->stretch, StretchType);
     return self;
@@ -152,6 +166,7 @@ Draw_font_style_eq(VALUE self, VALUE style)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(style, draw->info->style, StyleType);
     return self;
@@ -171,6 +186,7 @@ Draw_font_weight_eq(VALUE self, VALUE weight)
     Draw *draw;
     WeightType w;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
 
     if (FIXNUM_P(weight))
@@ -233,6 +249,7 @@ Draw_gravity_eq(VALUE self, VALUE grav)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(grav, draw->info->gravity, GravityType);
 
@@ -248,6 +265,7 @@ Draw_pointsize_eq(VALUE self, VALUE pointsize)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     draw->info->pointsize = NUM2DBL(pointsize);
     return self;
@@ -266,10 +284,11 @@ Draw_rotation_eq(VALUE self, VALUE deg)
     double degrees;
     AffineMatrix affine, current;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
 
     degrees = NUM2DBL(deg);
-    if (degrees != 0.0)
+    if (fabs(degrees) > DBL_EPSILON)
     {
         affine.sx=1.0;
         affine.rx=0.0;
@@ -303,11 +322,27 @@ Draw_stroke_eq(VALUE self, VALUE stroke)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     Color_to_PixelPacket(&draw->info->stroke, stroke);
     return self;
 }
 
+/*
+    Method:     Draw#stroke_width=
+    Purpose:    stroke_width attribute writer
+*/
+VALUE
+Draw_stroke_width_eq(VALUE self, VALUE stroke_width)
+{
+  Draw *draw;
+
+  rm_check_frozen(self);
+  Data_Get_Struct(self, Draw, draw);
+  draw->info->stroke_width = NUM2DBL(stroke_width);
+  return self;
+}
+
 /*
     Method:     Draw#text_antialias=
     Purpose:    text_antialias attribute writer
@@ -317,6 +352,7 @@ Draw_text_antialias_eq(VALUE self, VALUE text_antialias)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     draw->info->text_antialias = RTEST(text_antialias);
     return self;
@@ -331,6 +367,7 @@ Draw_undercolor_eq(VALUE self, VALUE undercolor)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     Color_to_PixelPacket(&draw->info->undercolor, undercolor);
     return self;
@@ -369,9 +406,10 @@ VALUE Draw_annotate(
     // allowing the app a chance to modify the object's attributes
     if (rb_block_given_p())
     {
-        rb_obj_instance_eval(0, NULL, self);
+        (void)rb_obj_instance_eval(0, NULL, self);
     }
 
+    rm_check_frozen(ImageList_cur_image(image_arg));
     Data_Get_Struct(ImageList_cur_image(image_arg), Image, image);
 
     // Copy text to Draw structure
@@ -406,11 +444,31 @@ VALUE Draw_annotate(
 
     draw->info->affine = keep;
 
-    HANDLE_IMG_ERROR(image)
+    HANDLE_ERROR_IMG(image)
 
     return self;
 }
 
+
+/*
+    Method:     Draw#clone
+    Notes:      see dup, init_copy
+*/
+VALUE
+Draw_clone(VALUE self)
+{
+    volatile VALUE clone;
+
+    clone = Draw_dup(self);
+    if (OBJ_FROZEN(self))
+    {
+        (void)rb_obj_freeze(clone);
+    }
+
+    return clone;
+}
+
+
 /*
     Method:     Draw#composite(x,y,width,height,img,operator=OverCompositeOp)
     Purpose:    Implement the "image" drawing primitive
@@ -426,6 +484,7 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     CompositeOperator cop = OverCompositeOp;
     volatile VALUE image;
     Image *comp_img;
+    struct TmpFile_Name *tmpfile;
     char name[MaxTextExtent];
                             // Buffer for "image" primitive
     char primitive[MaxTextExtent];
@@ -443,12 +502,8 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     // The default composition operator is "Over".
     if (argc == 6)
     {
-        if (TYPE(argv[5]) != T_FIXNUM)
-        {
-            rb_raise(rb_eTypeError, "composite operator must be a Fixnum (%s given)",
-                                rb_class2name(CLASS_OF(argv[5])));
-        }
         VALUE_TO_ENUM(argv[5], cop, CompositeOperator);
+
         switch(cop)
         {
             case AddCompositeOp:
@@ -518,14 +573,15 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
 
     // Create a temp copy of the composite image
     Data_Get_Struct(image, Image, comp_img);
-    write_temp_image(comp_img, name);
+    rm_write_temp_image(comp_img, name);
 
-    // Add the temp filename to the filename array
-    if (!draw->tmpfile_ary)
-    {
-        draw->tmpfile_ary = rb_ary_new();
-    }
-    rb_ary_push(draw->tmpfile_ary, rb_str_new2(name));
+    // Add the temp filename to the filename array.
+    // Use Magick storage since we need to keep the list around
+    // until destroy_Draw is called.
+    tmpfile = magick_malloc(sizeof(struct TmpFile_Name)+strlen(name));
+    strcpy(tmpfile->name, name);
+    tmpfile->next = draw->tmpfile_ary;
+    draw->tmpfile_ary = tmpfile;
 
     // Form the drawing primitive
     (void) sprintf(primitive, "image %s %g,%g,%g,%g '%s'", op, x, y, width, height, name);
@@ -554,13 +610,14 @@ Draw_draw(VALUE self, VALUE image_arg)
         rb_raise(rb_eArgError, "nothing to draw");
     }
 
+    rm_check_frozen(ImageList_cur_image(image_arg));
     Data_Get_Struct(ImageList_cur_image(image_arg), Image, image);
 
     // Point the DrawInfo structure at the current set of primitives.
     magick_clone_string(&(draw->info->primitive), STRING_PTR(draw->primitives));
 
     (void) DrawImage(image, draw->info);
-    HANDLE_IMG_ERROR(image)
+    HANDLE_ERROR_IMG(image)
 
     magick_free(draw->info->primitive);
     draw->info->primitive = NULL;
@@ -568,139 +625,84 @@ Draw_draw(VALUE self, VALUE image_arg)
     return self;
 }
 
+
 /*
- *  Method:     Draw#dup
- *  Purpose:    Copy a Draw object
- *  Notes:      a.k.a clone
+    Methods:    Draw#dup
+    Purpose:    Copy a Draw object
+    Notes:      Constructs a new Draw object, then calls initialize_copy
 */
 VALUE
 Draw_dup(VALUE self)
 {
-    Draw *draw, *draw2;
+    Draw *draw;
     volatile VALUE dup;
 
-    draw2 = ALLOC(Draw);
-    memset(draw2, '\0', sizeof(Draw));
-
-    Data_Get_Struct(self, Draw, draw);
-
-    draw2->info = CloneDrawInfo(NULL, draw->info);
-    if (!draw2->info)
+    draw = ALLOC(Draw);
+    memset(draw, 0, sizeof(Draw));
+    dup = Data_Wrap_Struct(CLASS_OF(self), mark_Draw, destroy_Draw, draw);
+    if (rb_obj_tainted(self))
     {
-        rb_raise(rb_eNoMemError, "not enough memory to continue");
+        (void)rb_obj_taint(dup);
     }
-
-    if (draw->primitives)
-    {
-        draw2->primitives = rb_str_dup(draw->primitives);
-    }
-    dup = Data_Wrap_Struct(CLASS_OF(self), mark_Draw, destroy_Draw, draw2);
-
-    return dup;
+    return rb_funcall(dup, ID_initialize_copy, 1, self);
 }
 
+
 /*
     Method:     Draw#get_type_metrics([image, ]text)
+                Draw#get_multiline_type_metrics([image, ]text)
     Purpose:    returns measurements for a given font and text string
     Notes:      If the image argument has been omitted, use a dummy
                 image, but make sure the text has none of the special
                 characters that refer to image attributes.
 */
-
-static VALUE get_dummy_tm_img(VALUE klass)
-{
-    volatile VALUE dummy_img = 0;
-    Info *info;
-    Image *image;
-
-    if (rb_cvar_defined(klass, _dummy_img__ID) != Qtrue)
-    {
-
-        info = CloneImageInfo(NULL);
-        if (!info)
-        {
-            rb_raise(rb_eNoMemError, "not enough memory to continue");
-        }
-        image = AllocateImage(info);
-        if (!image)
-        {
-            rb_raise(rb_eNoMemError, "not enough memory to continue");
-        }
-        DestroyImageInfo(info);
-        dummy_img = rm_image_new(image);
-
-        RUBY18(rb_cvar_set(klass, _dummy_img__ID, dummy_img, 0));
-        RUBY16(rb_cvar_set(klass, _dummy_img__ID, dummy_img));
-    }
-    dummy_img = rb_cvar_get(klass, _dummy_img__ID);
-
-    return dummy_img;
-}
-
-
 VALUE
 Draw_get_type_metrics(
     int argc,
     VALUE *argv,
     VALUE self)
 {
-    static char attrs[] = "bcdefghiklmnopqrstuwxyz";
-#define ATTRS_L (sizeof(attrs)-1)
-    Image *image;
-    Draw *draw;
-    TypeMetric metrics;
-    char *text;
-    long text_l;
-    int x;
-    unsigned int okay;
+    return get_type_metrics(argc, argv, self, GetTypeMetrics);
+}
 
-    switch (argc)
+
+VALUE
+Draw_get_multiline_type_metrics(
+     int argc,
+     VALUE *argv,
+     VALUE self)
+{
+#if defined(HAVE_GETMULTILINETYPEMETRICS)
+     return get_type_metrics(argc, argv, self, GetMultilineTypeMetrics);
+#else
+     rm_not_implemented();
+     return (VALUE)0;
+#endif
+}
+
+/*
+    Method:     Draw#initialize_copy
+    Purpose:    initialize clone, dup methods
+*/
+VALUE Draw_init_copy(VALUE self, VALUE orig)
+{
+    Draw *copy, *original;
+
+    Data_Get_Struct(orig, Draw, original);
+    Data_Get_Struct(self, Draw, copy);
+
+    copy->info = CloneDrawInfo(NULL, original->info);
+    if (!copy->info)
     {
-        case 1:                   // use default image
-            text = STRING_PTR_LEN(argv[0], text_l);
-
-            for (x = 0; x < text_l; x++)
-            {
-                // Ensure text string doesn't refer to image attributes.
-                if (text[x] == '%' && x < text_l-1)
-                {
-                    int y;
-                    char spec = text[x+1];
-
-                    for (y = 0; y < ATTRS_L; y++)
-                    {
-                        if (spec == attrs[y])
-                        {
-                            rb_raise(rb_eArgError,
-                                "text string contains image attribute reference `%%%c'",
-                                spec);
-                        }
-                    }
-                }
-            }
-
-            Data_Get_Struct(get_dummy_tm_img(CLASS_OF(self)), Image, image);
-            break;
-        case 2:
-            Data_Get_Struct(ImageList_cur_image(argv[0]), Image, image);
-            text = STRING_PTR(argv[1]);
-            break;                  // okay
-        default:
-            rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
-            break;
+        rb_raise(rb_eNoMemError, "not enough memory to continue");
     }
 
-    Data_Get_Struct(self, Draw, draw);
-    magick_clone_string(&draw->info->text, text);
-
-    okay = GetTypeMetrics(image, draw->info, &metrics);
-
-    if (!okay)
+    if (original->primitives)
     {
-        rb_warning("RMagick: get_type_metrics failed");
-        return Qnil;
+        copy->primitives = rb_str_dup(original->primitives);
     }
-    return TypeMetric_from_TypeMetric(&metrics);
+
+    return self;
 }
 
 /*
@@ -737,8 +739,8 @@ Draw_initialize(VALUE self)
     Data_Get_Struct(info_obj, Info, info);
     draw->info = CloneDrawInfo(info, NULL);
 
-    draw->primitives = 0;
-    draw->tmpfile_ary = 0;
+    draw->primitives = (VALUE)0;
+    draw->tmpfile_ary = NULL;
 
     return self;
 }
@@ -753,7 +755,7 @@ Draw_inspect(VALUE self)
     Draw *draw;
 
     Data_Get_Struct(self, Draw, draw);
-    return draw->primitives ? draw->primitives : rb_str_new2("");
+    return draw->primitives ? draw->primitives : rb_str_new2("(no primitives defined)");
 }
 
 /*
@@ -772,7 +774,7 @@ Draw_new(VALUE class)
     volatile VALUE draw_obj;
 
     draw = ALLOC(Draw);
-    memset(draw, '\0', sizeof(Draw));
+    memset(draw, 0, sizeof(Draw));
     draw_obj = Data_Wrap_Struct(class, mark_Draw, destroy_Draw, draw);
 
 #if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
@@ -792,9 +794,10 @@ Draw_primitive(VALUE self, VALUE primitive)
 {
     Draw *draw;
 
+    rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
 
-    if (!draw->primitives)
+    if (draw->primitives == (VALUE)0)
     {
         draw->primitives = primitive;
     }
@@ -816,11 +819,7 @@ mark_Draw(void *drawptr)
 {
     Draw *draw = (Draw *)drawptr;
 
-    if (draw->tmpfile_ary)
-    {
-        rb_gc_mark(draw->tmpfile_ary);
-    }
-    if (draw->primitives)
+    if (draw->primitives != (VALUE)0)
     {
         rb_gc_mark(draw->primitives);
     }
@@ -834,17 +833,17 @@ static void
 destroy_Draw(void *drawptr)
 {
     Draw *draw = (Draw *)drawptr;
-    volatile VALUE tmpfile;
+    struct TmpFile_Name *tmpfile;
 
     DestroyDrawInfo(draw->info);
 
     // Erase any temporary image files.
-    if (draw->tmpfile_ary)
+    while (draw->tmpfile_ary)
     {
-        while ((tmpfile = rb_ary_shift(draw->tmpfile_ary)) != Qnil)
-        {
-            delete_temp_image(STRING_PTR(tmpfile));
-        }
+        tmpfile = draw->tmpfile_ary;
+        draw->tmpfile_ary = draw->tmpfile_ary->next;
+        rm_delete_temp_image(tmpfile->name);
+        magick_free(tmpfile);
     }
 
     xfree(drawptr);
@@ -963,7 +962,7 @@ Montage_frame_eq(VALUE self, VALUE frame_arg)
     volatile VALUE frame;
 
     Data_Get_Struct(self, Montage, montage);
-    frame = rb_funcall(frame_arg, to_s_ID, 0);
+    frame = rb_funcall(frame_arg, ID_to_s, 0);
     magick_clone_string(&montage->info->frame, STRING_PTR(frame));
 
     return self;
@@ -980,7 +979,7 @@ Montage_geometry_eq(VALUE self, VALUE geometry_arg)
     volatile VALUE geometry;
 
     Data_Get_Struct(self, Montage, montage);
-    geometry = rb_funcall(geometry_arg, to_s_ID, 0);
+    geometry = rb_funcall(geometry_arg, ID_to_s, 0);
     magick_clone_string(&montage->info->geometry, STRING_PTR(geometry));
 
     return self;
@@ -1096,7 +1095,7 @@ destroy_Montage(void *obj)
     // If we saved a temporary texture image, delete it now.
     if (montage->info->texture != NULL)
     {
-        delete_temp_image(montage->info->texture);
+        rm_delete_temp_image(montage->info->texture);
     }
     DestroyMontageInfo(montage->info);
     xfree(montage);
@@ -1168,7 +1167,7 @@ Montage_texture_eq(VALUE self, VALUE texture)
     Data_Get_Struct(ImageList_cur_image(texture), Image, texture_image);
 
     // Write a temp copy of the image & save its name.
-    write_temp_image(texture_image, tmpnam);
+    rm_write_temp_image(texture_image, tmpnam);
     magick_clone_string(&montage->info->texture, tmpnam);
 
     return self;
@@ -1185,7 +1184,7 @@ Montage_tile_eq(VALUE self, VALUE tile_arg)
     volatile VALUE tile;
 
     Data_Get_Struct(self, Montage, montage);
-    tile = rb_funcall(tile_arg, to_s_ID, 0);
+    tile = rb_funcall(tile_arg, ID_to_s, 0);
     magick_clone_string(&montage->info->tile, STRING_PTR(tile));
 
     return self;
@@ -1205,3 +1204,105 @@ Montage_title_eq(VALUE self, VALUE title)
     return self;
 }
 
+
+static VALUE get_dummy_tm_img(VALUE klass)
+{
+    volatile VALUE dummy_img = 0;
+    Info *info;
+    Image *image;
+
+    if (rb_cvar_defined(klass, ID__dummy_img_) != Qtrue)
+    {
+
+        info = CloneImageInfo(NULL);
+        if (!info)
+        {
+            rb_raise(rb_eNoMemError, "not enough memory to continue");
+        }
+        image = AllocateImage(info);
+        if (!image)
+        {
+            rb_raise(rb_eNoMemError, "not enough memory to continue");
+        }
+        DestroyImageInfo(info);
+        dummy_img = rm_image_new(image);
+
+        RUBY18(rb_cvar_set(klass, ID__dummy_img_, dummy_img, 0));
+        RUBY16(rb_cvar_set(klass, ID__dummy_img_, dummy_img));
+    }
+    dummy_img = rb_cvar_get(klass, ID__dummy_img_);
+
+    return dummy_img;
+}
+
+
+/*
+ *  Static:     get_type_metrics
+ *  Purpose:    Call a get-type-metrics function
+ *  Notes:      called by Draw_get_type_metrics and Draw_get_multiline_type_metrics
+*/
+static VALUE
+get_type_metrics(
+     int argc,
+     VALUE *argv,
+     VALUE self,
+     get_type_metrics_func_t *getter)
+{
+     static char attrs[] = "bcdefghiklmnopqrstuwxyz";
+ #define ATTRS_L (sizeof(attrs)-1)
+     Image *image;
+     Draw *draw;
+     TypeMetric metrics;
+     char *text = NULL;
+     long text_l;
+     long x;
+     unsigned int okay;
+
+     switch (argc)
+     {
+         case 1:                   // use default image
+             text = STRING_PTR_LEN(argv[0], text_l);
+
+             for (x = 0; x < text_l; x++)
+             {
+                 // Ensure text string doesn't refer to image attributes.
+                 if (text[x] == '%' && x < text_l-1)
+                 {
+                     int y;
+                     char spec = text[x+1];
+
+                     for (y = 0; y < ATTRS_L; y++)
+                     {
+                         if (spec == attrs[y])
+                         {
+                             rb_raise(rb_eArgError,
+                                 "text string contains image attribute reference `%%%c'",
+                                 spec);
+                         }
+                     }
+                 }
+             }
+
+             Data_Get_Struct(get_dummy_tm_img(CLASS_OF(self)), Image, image);
+             break;
+         case 2:
+             Data_Get_Struct(ImageList_cur_image(argv[0]), Image, image);
+             text = STRING_PTR(argv[1]);
+             break;                  // okay
+         default:
+             rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
+             break;
+     }
+
+     Data_Get_Struct(self, Draw, draw);
+     magick_clone_string(&draw->info->text, text);
+
+     okay = (*getter)(image, draw->info, &metrics);
+
+     if (!okay)
+     {
+         rb_warning("RMagick: get_type_metrics failed");
+         return Qnil;
+     }
+     return TypeMetric_from_TypeMetric(&metrics);
+}
