@@ -1,4 +1,4 @@
-/* $Id: rmilist.c,v 1.10 2004/01/26 20:19:58 rmagick Exp $ */
+/* $Id: rmilist.c,v 1.20 2004/12/05 22:36:11 rmagick Exp $ */
 /*============================================================================\
 |                Copyright (C) 2004 by Timothy P. Hunter
 | Name:     rmilist.c
@@ -8,6 +8,7 @@
 
 #include "rmagick.h"
 
+
 /*
     Method:     ImageList#animate(<delay>)
     Purpose:    repeatedly display the images in the images array to an XWindow
@@ -23,7 +24,7 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
     volatile VALUE info_obj;
 
     // Convert the images array to an images sequence.
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
 
     if (argc > 1)
     {
@@ -47,8 +48,8 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
     Data_Get_Struct(info_obj, Info, info);
 
     (void) AnimateImages(info, images);
-    handle_all_errors(images);
-    unseq(images);
+    rm_handle_all_errors(images);
+    rm_split(images);
 
     return self;
 }
@@ -66,7 +67,7 @@ ImageList_append(VALUE self, VALUE stack_arg)
     ExceptionInfo exception;
 
     // Convert the image array to an image sequence.
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
 
     // If stack == true, stack rectangular images top-to-bottom,
     // otherwise left-to-right.
@@ -75,7 +76,7 @@ ImageList_append(VALUE self, VALUE stack_arg)
     GetExceptionInfo(&exception);
     result = AppendImages(images, stack, &exception);
     HANDLE_ERROR
-    unseq(images);
+    rm_split(images);
 
     return rm_image_new(result);
 }
@@ -92,12 +93,12 @@ ImageList_average(VALUE self)
     ExceptionInfo exception;
 
     // Convert the images array to an images sequence.
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
     result = AverageImages(images, &exception);
-    handle_all_errors(images);
-    unseq(images);
+    rm_handle_all_errors(images);
+    rm_split(images);
 
     return rm_image_new(result);
 }
@@ -113,64 +114,41 @@ ImageList_average(VALUE self)
 VALUE
 ImageList_coalesce(VALUE self)
 {
-    Image *images, *results, *result, *next;
-    volatile VALUE new_imagelist;
+    Image *images, *results;
     ExceptionInfo exception;
 
     // Convert the image array to an image sequence.
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
     results = CoalesceImages(images, &exception);
     HANDLE_ERROR
-    unseq(images);
+    rm_split(images);
 
-    new_imagelist = rm_imagelist_new();
-
-    // CoalesceImages returns an image sequence. Create a
-    // new ImageList and store the images in the array.
-#if HAVE_REMOVEFIRSTIMAGEFROMLIST
-    result = results;
-    while (result)
-    {
-        next = RemoveFirstImageFromList(&result);
-        rm_imagelist_push(new_imagelist, rm_image_new(next));
-    }
-#else
-    for (result = results; result; result = next)
-    {
-        next = GET_NEXT_IMAGE(result);
-        result->previous = result->next = NULL;
-        rm_imagelist_push(new_imagelist, rm_image_new(result));
-    }
-#endif
-
-    // Set new_imagelist.scene = 0
-    rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
-
-    return new_imagelist;
+    return rm_imagelist_from_images(results);
 }
 
+
 /*
     Method:     ImageList#deconstruct
     Purpose:    compares each image with the next in a sequence and returns
                 the maximum bounding region of any pixel differences it
                 discovers.
-    Returns:    a new image, or nil
+    Returns:    a new imagelist
 */
 VALUE
 ImageList_deconstruct(VALUE self)
 {
-    Image *images, *new_image;
+    Image *new_images, *images;
     ExceptionInfo exception;
 
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
-    new_image = DeconstructImages(images, &exception);
+    new_images = DeconstructImages(images, &exception);
     HANDLE_ERROR
-    unseq(images);
+    rm_split(images);
 
-    return rm_image_new(new_image);
+    return rm_imagelist_from_images(new_images);
 }
 
 /*
@@ -186,7 +164,7 @@ ImageList_display(VALUE self)
     unsigned int ok;
 
     // Convert the images array to an images sequence.
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
 
     // Create a new Info object to use with this call
     info_obj = rm_info_new();
@@ -195,9 +173,9 @@ ImageList_display(VALUE self)
     ok = DisplayImages(info, images);
     if (!ok)
     {
-        handle_all_errors(images);
+        rm_handle_all_errors(images);
     }
-    unseq(images);
+    rm_split(images);
 
     return self;
 }
@@ -214,11 +192,11 @@ ImageList_flatten_images(VALUE self)
     Image *images, *new_image;
     ExceptionInfo exception;
 
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
     new_image = FlattenImages(images, &exception);
     HANDLE_ERROR
-    unseq(images);
+    rm_split(images);
 
     return rm_image_new(new_image);
 }
@@ -227,16 +205,15 @@ ImageList_flatten_images(VALUE self)
 /*
     Method:     ImageList#map
     Purpose:    Call MapImages
-    Returns:    a new Image with mapped images. @scene is set to self.scene
+    Returns:    a new ImageList with mapped images. @scene is set to self.scene
 */
 VALUE
 ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
 {
-    Image *images, *new_images = NULL;
-    Image *new_image;
+    Image *images, *clone_images = NULL;
     Image *map;
     unsigned int dither;
-    volatile VALUE image, image_obj, scene;
+    volatile VALUE image, scene, new_imagelist;
     ExceptionInfo exception;
 
     image = ImageList_cur_image(map_image);
@@ -248,31 +225,23 @@ ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
     }
 
     // Convert image array to image sequence, clone image sequence.
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
-    new_images = CloneImageList(images, &exception);
+    clone_images = CloneImageList(images, &exception);
     HANDLE_ERROR
-    unseq(images);
+    rm_split(images);
 
     // Call ImageMagick
     dither = !(dither_arg == Qfalse || dither_arg == Qnil);
-    (void) MapImages(new_images, map, dither);
-    HANDLE_IMG_ERROR(new_images)
-
-    // Create new ImageList object, convert mapped image sequence to images, append
-    // to imagelist.
-    image_obj = rm_imagelist_new();
-    while ((new_image = ShiftImageList(&new_images)))
-    {
-        rm_imagelist_push(image_obj, rm_image_new(new_image));
-    }
-
+    (void) MapImages(clone_images, map, dither);
+    HANDLE_ERROR_IMG(clone_images)
 
     // Set @scene in new ImageList object to same value as in self.
+    new_imagelist = rm_imagelist_from_images(clone_images);
     scene = rb_iv_get(self, "@scene");
-    rb_iv_set(image_obj, "@scene", scene);
+    (void) rm_imagelist_scene_eq(new_imagelist, scene);
 
-    return image_obj;
+    return new_imagelist;
 }
 
 /*
@@ -286,8 +255,7 @@ ImageList_montage(VALUE self)
 {
     volatile VALUE montage_obj;
     Montage *montage;
-    Image *montage_seq, *next, *image, *image_list;
-    volatile VALUE new_imagelist;
+    Image *montage_seq, *image_list;
     ExceptionInfo exception;
 
     // Create a new instance of the Magick::Montage class
@@ -301,7 +269,7 @@ ImageList_montage(VALUE self)
 
     Data_Get_Struct(montage_obj, Montage, montage);
 
-    image_list = toseq(self);
+    image_list = rm_images_from_imagelist(self);
 
     // If app specified a non-default composition operator, use it for all images.
     if (montage->compose != UndefinedCompositeOp)
@@ -318,21 +286,9 @@ ImageList_montage(VALUE self)
     // MontageImage can return more than one image.
     montage_seq = MontageImages(image_list, montage->info, &exception);
     HANDLE_ERROR
-    unseq(image_list);
+    rm_split(image_list);
 
-    // Construct a new image and store the image(s) in its images array.
-    new_imagelist = rm_imagelist_new();
-    for (image = montage_seq; image; image = next)
-    {
-        next = GET_NEXT_IMAGE(image);
-        image->previous = image->next = NULL;
-        rm_imagelist_push(new_imagelist, rm_image_new(image));
-    }
-
-    // img.scene = 0
-    rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
-
-    return new_imagelist;
+    return rm_imagelist_from_images(montage_seq);
 }
 
 /*
@@ -346,9 +302,8 @@ ImageList_montage(VALUE self)
 VALUE
 ImageList_morph(VALUE self, VALUE nimages)
 {
-    Image *images, *new_images, *next, *new;
+    Image *images, *new_images;
     ExceptionInfo exception;
-    volatile VALUE new_imagelist;
     unsigned long number_images;
 
     if (rm_imagelist_length(self) < 1)
@@ -361,23 +316,12 @@ ImageList_morph(VALUE self, VALUE nimages)
         rb_raise(rb_eArgError, "number of intervening images must be > 0");
     }
 
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
     new_images = MorphImages(images, number_images, &exception);
     HANDLE_ERROR
 
-    new_imagelist = rm_imagelist_new();
-    for (new = new_images; new; new = next)
-    {
-        next = GET_NEXT_IMAGE(new);
-        new->previous = new->next = NULL;
-        rm_imagelist_push(new_imagelist, rm_image_new(new));
-    }
-
-    // new_imagelist.scene = 0
-    rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
-
-    return new_imagelist;
+    return rm_imagelist_from_images(new_images);
 }
 
 /*
@@ -391,23 +335,122 @@ ImageList_mosaic(VALUE self)
     Image *images, *new_image;
     ExceptionInfo exception;
 
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
     new_image = MosaicImages(images, &exception);
     HANDLE_ERROR
-    unseq(images);
+    rm_split(images);
 
     return rm_image_new(new_image);
 }
 
 /*
     External:   rm_imagelist_new
-    Purpose:    create a new ImageList object
+    Purpose:    create a new ImageList object with no images
+    Notes:      this simply calls ImageList.new() in RMagick.rb
 */
 VALUE
 rm_imagelist_new()
 {
-    return rb_funcall(Class_ImageList, new_ID, 0);
+    return rb_funcall(Class_ImageList, ID_new, 0);
+}
+
+
+
+/*
+     Extern:   rm_imagelist_from_images
+     Purpose:  Construct a new imagelist object from a list of images
+     Notes:    Sets @scene to 0.
+*/
+VALUE
+rm_imagelist_from_images(Image *images)
+{
+     volatile VALUE new_imagelist;
+#if defined(HAVE_REMOVEFIRSTIMAGEFROMLIST)
+     Image *image;
+
+     new_imagelist = rm_imagelist_new();
+
+     while (images)
+     {
+          image = RemoveFirstImageFromList(&images);
+          rm_imagelist_push(new_imagelist, rm_image_new(image));
+     }
+#else
+     Image *image, *next;
+
+     new_imagelist = rm_imagelist_new();
+
+     for (image = images; image; image = next)
+     {
+          next = GET_NEXT_IMAGE(image);
+          image->previous = image->next = NULL;
+          rm_imagelist_push(new_imagelist, rm_image_new(image));
+     }
+#endif
+
+     rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
+     return new_imagelist;
+}
+
+
+/*
+    Extern:     rm_images_from_imagelist
+    Purpose:    Convert an array of Image *s to an ImageMagick scene
+                sequence (i.e. a doubly-linked list of Images)
+    Returns:    a pointer to the head of the scene sequence list
+*/
+Image *
+rm_images_from_imagelist(VALUE imagelist)
+{
+    long x, len;
+    Image *head = NULL;
+#if !defined(HAVE_APPENDIMAGETOLIST)
+    Image *tail = NULL;
+#endif
+
+    Check_Type(imagelist, T_ARRAY);
+    len = rm_imagelist_length(imagelist);
+    if (len == 0)
+    {
+        rb_raise(rb_eArgError, "no images in this image list");
+    }
+
+    for (x = 0; x < len; x++)
+    {
+        Image *image;
+
+        Data_Get_Struct(rb_ary_entry(imagelist, x), Image, image);
+#if defined(HAVE_APPENDIMAGETOLIST)
+        AppendImageToList(&head, image);
+#else
+        if (!head)
+        {
+            head = image;
+        }
+        else
+        {
+            image->previous = tail;
+            tail->next = image;
+        }
+        tail = image;
+#endif
+    }
+
+    return head;
+}
+
+
+/*
+ *   Extern:   rm_imagelist_scene_eq(imagelist, scene)
+ *   Purpose:  @scene attribute writer
+*/
+VALUE
+rm_imagelist_scene_eq(VALUE imagelist, VALUE scene)
+{
+     rm_check_frozen(imagelist);
+     rb_iv_set(imagelist, "@scene", scene);
+     return scene;
 }
 
 /*
@@ -419,7 +462,7 @@ rm_imagelist_length(VALUE imagelist)
 {
     volatile VALUE len;
 
-    len = rb_funcall(imagelist, length_ID, 0);
+    len = rb_funcall(imagelist, ID_length, 0);
     return FIX2INT(len);
 }
 
@@ -427,10 +470,11 @@ rm_imagelist_length(VALUE imagelist)
     External:   rm_imagelist_push
     Purpose:    push an image onto the end of the imagelist
 */
-VALUE
+void
 rm_imagelist_push(VALUE imagelist, VALUE image)
 {
-    return rb_funcall(imagelist, push_ID, 1, image);
+    rm_check_frozen(imagelist);
+    (void) rb_funcall(imagelist, ID_push, 1, image);
 }
 
 /*
@@ -477,10 +521,10 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
 
     // Convert image array to image sequence, clone image sequence.
     GetExceptionInfo(&exception);
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
     new_images = CloneImageList(images, &exception);
     HANDLE_ERROR
-    unseq(images);
+    rm_split(images);
 
     QuantizeImages(&quantize_info, new_images);
 
@@ -519,7 +563,7 @@ ImageList_to_blob(VALUE self)
     Data_Get_Struct(info_obj, Info, info);
 
     // Convert the images array to an images sequence.
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
     (void) SetImageInfo(info, True, &exception);
@@ -540,11 +584,23 @@ ImageList_to_blob(VALUE self)
     GetExceptionInfo(&exception);
     blob = ImageToBlob(info, images, &length, &exception);
     HANDLE_ERROR
-    unseq(images);
+    rm_split(images);
 
     return (blob && length) ? rb_str_new(blob, length) : Qnil;
 }
 
+
+/*
+ *  Static:     file_arg_rescue
+ *  Purpose:    called when `arg_to_str' raised an exception
+*/
+static VALUE file_arg_rescue(VALUE arg)
+{
+    rb_raise(rb_eTypeError, "argument must be path name or open file (%s given)",
+            rb_class2name(CLASS_OF(arg)));
+}
+
+
 /*
   Method:   ImageList#write(file)
   Purpose:  Write all the images to the specified file. If the file format
@@ -570,18 +626,9 @@ ImageList_write(VALUE self, VALUE file)
 
 
     // Convert the images array to an images sequence.
-    images = toseq(self);
+    images = rm_images_from_imagelist(self);
 
-    // Copy the filename to the Info and to the Image.
-    if (TYPE(file) == T_STRING)
-    {
-        filename = STRING_PTR_LEN(file, filenameL);
-        filenameL = min(filenameL, MaxTextExtent-1);
-        memcpy(info->filename, filename, (size_t)filenameL);
-        info->filename[filenameL] = '\0';
-        info->file = NULL;
-    }
-    else if (TYPE(file) == T_FILE)
+    if (TYPE(file) == T_FILE)
     {
         OpenFile *fptr;
 
@@ -591,8 +638,15 @@ ImageList_write(VALUE self, VALUE file)
     }
     else
     {
-        rb_raise(rb_eTypeError, "argument must be String or File (%s given)",
-                rb_class2name(CLASS_OF(file)));
+        // Convert arg to string. Catch exceptions.
+        file = rb_rescue(rm_obj_to_s, file, file_arg_rescue, file);
+
+        // Copy the filename to the Info and to the Image.
+        filename = STRING_PTR_LEN(file, filenameL);
+        filenameL = min(filenameL, MaxTextExtent-1);
+        memcpy(info->filename, filename, (size_t)filenameL);
+        info->filename[filenameL] = '\0';
+        info->file = NULL;
     }
 
     // Copy the filename into each images. Set a scene number to be used if
@@ -619,13 +673,13 @@ ImageList_write(VALUE self, VALUE file)
     for (img = images; img; img = GET_NEXT_IMAGE(img))
     {
         (void) WriteImage(info, img);
-        handle_all_errors(images);
+        rm_handle_all_errors(images);
         if (info->adjoin)
         {
             break;
         }
     }
 
-    unseq(images);
+    rm_split(images);
     return self;
 }
